@@ -17,8 +17,10 @@
  ***************************************************************************/
 
 /****************************************************************************
-*  Header Files
-****************************************************************************/
+ *  Header Files
+ ****************************************************************************/
+#include <assert.h>
+
 #include "./vpx_scale_rtcd.h"
 #include "vpx_mem/vpx_mem.h"
 #include "vpx_scale/vpx_scale.h"
@@ -172,6 +174,7 @@ static void scale1d_c(const unsigned char *source, int source_step,
   /*assert ( (source_length-1) * dest_scale >= (dest_length-1) * source_scale
    * );*/
 
+  assert(dest_scale != 0);
   for (i = 0; i < dest_length * dest_step; i += dest_step) {
     dest[i] = (char)((left_modifier * left_pixel +
                       right_modifier * right_pixel + round_value) /
